"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const ioserver = require('socket.io');
const ioclient = require('socket.io-client');
class DeclaredMethod {
    constructor(name, code) {
        this.name = name;
        this.code = code;
    }
}
class RemoteError extends Error {
    constructor(thrownException) {
        super(thrownException.message);
        this.name = thrownException.remote_name;
        this.message = thrownException.remote_message;
        this.stack = thrownException.remote_stack;
    }
}
function S4() {
    return (((1 + Math.random()) * 0x10000) | 0).toString(16).substring(1);
}
function GUID() {
    return `${S4()}${S4()}-${S4()}-4${S4().substr(0, 3)}-${S4()}-${S4()}${S4()}${S4()}`;
}
class SioRpcServer {
    constructor(httpServer) {
        this.io = ioserver(httpServer);
        this.connectedSockets = [];
        this.declaredMethods = [];
    }
    declare(methodName, methodCode) {
        this.declaredMethods.push(new DeclaredMethod(methodName, (returnFunction, args) => __awaiter(this, void 0, void 0, function* () {
            let result;
            try {
                result = yield methodCode.apply(null, args);
            }
            catch (exception) {
                return returnFunction(undefined, exception);
            }
            return returnFunction(result, undefined);
        })));
    }
    publish(eventName, ...args) {
        for (const connectedSocket of this.connectedSockets) {
            connectedSocket.emit(`${eventName}..event`, args);
        }
    }
    run() {
        this.io.on('connection', (socket) => {
            this.connectedSockets.push(socket);
            socket.on('disconnect', () => {
                const socketIndex = this.connectedSockets.indexOf(socket);
                if (socketIndex !== -1) {
                    this.connectedSockets.splice(socketIndex, 1);
                }
            });
            for (const declaredMethod of this.declaredMethods) {
                socket.on(`${declaredMethod.name}..call`, (cookieAndArgs) => {
                    const returnMethod = (value, exception) => {
                        const returnedValue = value;
                        let thrownException;
                        if (typeof exception !== 'undefined') {
                            thrownException = {
                                remote_name: exception.name,
                                remote_message: exception.message,
                                remote_stack: exception.stack,
                            };
                        }
                        const returnedValueAndThrownException = { returnedValue, thrownException };
                        socket.emit(`${declaredMethod.name}..return..${cookieAndArgs.cookie}`, returnedValueAndThrownException);
                    };
                    declaredMethod.code(returnMethod, cookieAndArgs.args);
                });
            }
        });
    }
}
exports.SioRpcServer = SioRpcServer;
class SioRpcClient {
    constructor(url) {
        this.socket = ioclient.connect(url);
    }
    call(methodName, ...args) {
        return new Promise((resolve, reject) => {
            const cookie = GUID();
            this.socket.once(`${methodName}..return..${cookie}`, (returnedValueAndThrownException) => {
                if (typeof returnedValueAndThrownException.thrownException !== 'undefined') {
                    return reject(new RemoteError(returnedValueAndThrownException.thrownException));
                }
                resolve(returnedValueAndThrownException.returnedValue);
            });
            this.socket.emit(`${methodName}..call`, { cookie, args });
        });
    }
    subscribe(eventName, eventCode) {
        this.socket.on(`${eventName}..event`, (args) => {
            eventCode.apply(null, args);
        });
    }
    subscribeOnce(eventName, eventCode) {
        this.socket.once(`${eventName}..event`, (args) => {
            eventCode.apply(null, args);
        });
    }
}
exports.SioRpcClient = SioRpcClient;
//# sourceMappingURL=main.js.map